package com.qs.mp.web.controller.api.admin;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.qs.mp.admin.domain.CouponPkg;
import com.qs.mp.admin.domain.param.CouponPkgParam;
import com.qs.mp.admin.domain.vo.CouponPkgVO;
import com.qs.mp.admin.service.ICouponPkgItemService;
import com.qs.mp.admin.service.ICouponPkgService;
import com.qs.mp.common.core.domain.AjaxResult;
import com.qs.mp.common.core.page.TableDataInfo;
import com.qs.mp.web.controller.common.BaseApiController;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 券包管理API
 *
 * @author Cup
 * @date 2022/5/6
 */
@Api(tags = "券包管理API")
@RestController
@RequestMapping("/api/v1/mp/admin/coupon/pkg/*")
public class CouponPkgMgrController extends BaseApiController {


    @Autowired
    private ICouponPkgService couponPkgService;



    @PostMapping("/create")
    @ApiOperation("创建券包")
    public AjaxResult create(@RequestBody CouponPkgParam couponPkgParam) {

        couponPkgService.create(couponPkgParam);

        return AjaxResult.success();
    }


    @PostMapping("/update")
    @ApiOperation("更新券包")
    public AjaxResult update(@RequestBody CouponPkgParam couponPkgParam) {

        couponPkgService.updateCouponPkg(couponPkgParam);

        return AjaxResult.success();
    }

    @PostMapping("/list")
    @ApiOperation("券包列表")
    @ApiResponses(
            @ApiResponse(code = 200, message = "券包列表", response = CouponPkg.class)
    )
    public TableDataInfo list() {
        startPage();
        List<CouponPkg> list = couponPkgService.list(new LambdaQueryWrapper<CouponPkg>().orderByDesc(CouponPkg::getId));
        return getDataTable(list);
    }

    @PostMapping("/detail/{id}")
    @ApiOperation("券包详情")
    @ApiResponses(
            @ApiResponse(code = 200, message = "券包详情", response = CouponPkgVO.class)
    )
    public AjaxResult detail(@PathVariable("id") Long id) {

        return AjaxResult.success(couponPkgService.detail(id));
    }

}
